--- old/usr/src/cmd/zpool/zpool_main.c	2013-04-23 15:34:30.677578102 -0600
+++ new/usr/src/cmd/zpool/zpool_main.c	2013-04-23 15:34:30.686809864 -0600
@@ -677,6 +677,7 @@
 				goto errout;
 			break;
 		case 'm':
+			/* Equivalent to -O mountpoint=optarg */
 			mountpoint = optarg;
 			break;
 		case 'o':
@@ -715,6 +716,14 @@
 			*propval = '\0';
 			propval++;
 
+			/*
+			 * Mountpoints are checked and then added later.
+			 * Uniquely among properties, they can be specified
+			 * more than once, to avoid conflict with -m.
+			 */
+			if (!strcmp(optarg,
+			    zfs_prop_to_name(ZFS_PROP_MOUNTPOINT)))
+				mountpoint = propval;
 			if (add_prop_list(optarg, propval, &fsprops, B_FALSE))
 				goto errout;
 			break;
@@ -833,6 +842,15 @@
 		}
 	}
 
+	/*
+	 * Now that the mountpoint's validity has been checked, ensure that
+	 * the property is set appropriately prior to creating the pool.
+	 */
+	if (mountpoint != NULL)
+		if (add_prop_list(zfs_prop_to_name(ZFS_PROP_MOUNTPOINT),
+		    mountpoint, &fsprops, B_FALSE))
+			goto errout;
+
 	if (dryrun) {
 		/*
 		 * For a dry run invocation, print out a basic message and run
@@ -877,11 +895,6 @@
 			zfs_handle_t *pool = zfs_open(g_zfs, poolname,
 			    ZFS_TYPE_FILESYSTEM);
 			if (pool != NULL) {
-				if (mountpoint != NULL)
-					verify(zfs_prop_set(pool,
-					    zfs_prop_to_name(
-					    ZFS_PROP_MOUNTPOINT),
-					    mountpoint) == 0);
 				if (zfs_mount(pool, NULL, 0) == 0)
 					ret = zfs_shareall(pool);
 				zfs_close(pool);
--- old/usr/src/lib/libzfs/common/libzfs_pool.c	2013-04-23 15:34:31.153028682 -0600
+++ new/usr/src/lib/libzfs/common/libzfs_pool.c	2013-04-23 15:34:31.162284805 -0600
@@ -1179,21 +1179,6 @@
 		}
 	}
 
-	/*
-	 * If this is an alternate root pool, then we automatically set the
-	 * mountpoint of the root dataset to be '/'.
-	 */
-	if (nvlist_lookup_string(props, zpool_prop_to_name(ZPOOL_PROP_ALTROOT),
-	    &altroot) == 0) {
-		zfs_handle_t *zhp;
-
-		verify((zhp = zfs_open(hdl, pool, ZFS_TYPE_DATASET)) != NULL);
-		verify(zfs_prop_set(zhp, zfs_prop_to_name(ZFS_PROP_MOUNTPOINT),
-		    "/") == 0);
-
-		zfs_close(zhp);
-	}
-
 create_failed:
 	zcmd_free_nvlists(&zc);
 	nvlist_free(zc_props);
